@@ -1,7 +1,6 @@
 import os
 import sys
 import copy
-import re
 import logging
 import numpy as np
 import pandas as pd
@@ -13,11 +12,13 @@ from time import strftime, perf_counter, gmtime
 from prettytable import PrettyTable
 # My own module!
 import cvnn.layers as layers
+from cvnn.optimizers import get_optimizer
 import cvnn.dataset as dp
 import cvnn.data_analysis as da
 from cvnn.utils import create_folder
 from cvnn import logger
 
+
 try:
     import cPickle as pickle
 except ImportError:
@@ -48,13 +49,16 @@ class CvnnModel:
     # Constructor and Stuff
     # =====================
 
-    def __init__(self, name, shape, loss_fun, verbose=True, tensorboard=True):
+    def __init__(self, name, shape, loss_fun, optimizer='Adam',
+                 verbose: bool = True, tensorboard: bool = True):
         """
         Constructor
         :param name: Name of the model. It will be used to distinguish models
         :param shape: List of cvnn.layers.ComplexLayer objects
         :param loss_fun: tensorflow.python.keras.losses to be used.
-        :param verbose: if True it will print information of the model just created
+        :param optimizer: Optimizer to be used. Keras optimizers are not allowed.
+            Can be either cvnn.optimizers.Optimizer or a string listed in opt_dispatcher.
+        :param verbose: if True it will print information of np.prod(w_vals.shape)the model just created
         :param tensorboard: If true it will save tensorboard information inside log/.../tensorboard_logs/
                 - Loss and accuracy
                 - Graph
@@ -65,8 +69,12 @@ class CvnnModel:
         if not all([isinstance(layer, layers.ComplexLayer) for layer in shape]):
             logger.error("All layers in shape must be a cvnn.layer.Layer", exc_info=True)
             sys.exit(-1)
+        layers.ComplexLayer.last_layer_output_dtype = None
+        layers.ComplexLayer.last_layer_output_size = None
         self.shape = shape
         self.loss_fun = loss_fun
+        self.optimizer = get_optimizer(optimizer)       # This checks input already
+        self.optimizer.compile(shape=shape)
         self.epochs_done = 0
         self.run_pandas = pd.DataFrame(columns=['step', 'epoch',
                                                 'train loss', 'train accuracy', 'test loss', 'test accuracy'])
@@ -155,7 +163,8 @@ class CvnnModel:
             else:
                 logger.error("Layer " + str(layer) + " not child of cvnn.layers.ComplexLayer")
                 sys.exit(-1)
-        return CvnnModel(self.name, new_shape, self.loss_fun, verbose=False, tensorboard=self.tensorboard)
+        return CvnnModel(self.name, new_shape, self.loss_fun, optimizer=self.optimizer.__deepcopy__(),
+                         verbose=False, tensorboard=self.tensorboard)
 
     def _get_real_equivalent_multiplier(self, classifier: bool = True, capacity_equivalent: bool = True,
                                         equiv_technique: str = 'ratio'):
@@ -293,7 +302,7 @@ class CvnnModel:
         if name is None:
             name = self.name + "_real_equiv"
         # set_trace()
-        return CvnnModel(name=name, shape=real_shape, loss_fun=self.loss_fun,
+        return CvnnModel(name=name, shape=real_shape, loss_fun=self.loss_fun, optimizer=self.optimizer.__deepcopy__(),
                          tensorboard=self.tensorboard, verbose=False)
 
     # ====================
@@ -308,19 +317,13 @@ class CvnnModel:
         # https://stackoverflow.com/questions/4103773/efficient-way-of-having-a-function-only-execute-once-in-a-loop
         tf.summary.trace_on(graph=True, profiler=True)  # https://www.tensorflow.org/tensorboard/graphs
 
-    @run_once
-    def _end_graph_tensorflow(self):
-        with self.graph_writer.as_default():
-            tf.summary.trace_export(name="graph", step=0, profiler_outdir=self.graph_writer_logdir)
-
     # Add '@tf.function' to accelerate the code by a lot!
     @tf.function
-    def _train_step(self, x_train_batch, y_train_batch, learning_rate):
+    def _train_step(self, x_train_batch, y_train_batch):
         """
         Performs one step of the training
         :param x_train_batch: input
         :param y_train_batch: labels
-        :param learning_rate: learning rate fot the gradient descent
         :return: None
         """
         with tf.GradientTape() as tape:
@@ -340,12 +343,15 @@ class CvnnModel:
 
         # Backpropagation
         with tf.name_scope("Optimizer"):
-            for i, val in enumerate(variables):
-                val.assign(val - learning_rate * gradients[i])  # TODO: For the moment the optimization is only GD
+            self.optimizer.optimize(variables=variables, gradients=gradients)
+
+    @run_once
+    def _end_graph_tensorflow(self):
+        with self.graph_writer.as_default():
+            tf.summary.trace_export(name="graph", step=0, profiler_outdir=self.graph_writer_logdir)
 
     def fit(self, x, y=None,
-            validation_split=0.0, validation_data=None,
-            learning_rate: float = 0.01, epochs: int = 10, batch_size: int = 32,
+            validation_split=0.0, validation_data=None, epochs: int = 10, batch_size: int = 32,
             verbose=True, display_freq: int = 1,
             save_model_checkpoints=False, save_csv_history=True, shuffle=True):
         """
@@ -366,7 +372,6 @@ class CvnnModel:
             It can be:
                 - tuple (x_val, y_val) of Numpy arrays or tensors. Preferred data type (less overhead).
                 - A tf.data dataset.
-        :param learning_rate: Learning rate for the gradient descent. For the moment only SGD is supported.
         :param epochs: (uint) Number of epochs to do.
         :param batch_size: (uint) Batch size of the data. Default 32 (because keras use 32 so... why not?)
         :param verbose: Verbosity Mode
@@ -386,21 +391,22 @@ class CvnnModel:
                 https://www.tensorflow.org/api_docs/python/tf/keras/Model#fit
         :param display_freq: Integer (Default 1)
             Frequency on terms of epochs before saving information and running a checkpoint.
-        :param save_model_checkpoints: Save the model to be able to load and continue training later (Not yet working)
-        :param save_csv_history: Save information of the train and test loss and accuracy on csv files.
+        :param save_model_checkpoints: (Boolean)
+                    Save the model to be able to load and continue training later TODO: Not yet working
+        :param save_csv_history: (Boolean) Save information of the train and test loss and accuracy on csv files.
         :param shuffle: (Boolean) Whether to shuffle the training data before each epoch. Default: True
         :return: None
         """
         # Check input
-        verbose = self.verify_fit_input(save_model_checkpoints, epochs, batch_size, learning_rate, display_freq, verbose)
+        verbose = self.verify_fit_input(save_model_checkpoints, epochs, batch_size, display_freq, verbose)
         # Prepare dataset
-        train_dataset, (x_test, y_test) = self._process_dataset(x, y, validation_split, validation_data)
-        train_dataset = train_dataset.batch(batch_size=batch_size)  # TODO: Check if batch_size = 1
+        train_dataset, (x_test, y_test) = self._process_dataset(x, y, validation_split, validation_data, batch_size)
+        # train_dataset = train_dataset.batch(batch_size=batch_size)  # TODO: Check if batch_size = 1
         # Print start condition
         start_status = ''
         if x_test is not None and y_test is not None:
             start_status = self._get_str_evaluate(self.epochs_done, epochs, x_test, y_test)
-        self._manage_string("Starting training...\nLearning rate = " + str(learning_rate) + "\n" +
+        self._manage_string("Starting training...\n" +
                             "Epochs = " + str(epochs) + "\nBatch Size = " + str(batch_size) + "\n" +
                             start_status, verbose)
         # -----------------------------------------------------
@@ -429,7 +435,7 @@ class CvnnModel:
                 iteration += 1
                 # Run optimization op (backpropagation)
                 self._start_graph_tensorflow()
-                self._train_step(x_batch, y_batch, learning_rate)
+                self._train_step(x_batch, y_batch)
                 self._end_graph_tensorflow()
             if total_iteration is None:
                 total_iteration = iteration
@@ -469,8 +475,7 @@ class CvnnModel:
                             verbose)
         self.plotter.reload_data()
 
-    def verify_fit_input(self, save_model_checkpoints, epochs: int, batch_size: int,
-                         learning_rate: float, display_freq: int, verbose):
+    def verify_fit_input(self, save_model_checkpoints, epochs: int, batch_size: int, display_freq: int, verbose):
         assert not save_model_checkpoints  # TODO: Not working for the moment, sorry!
         if not (isinstance(epochs, int) and epochs > 0):
             logger.error("Epochs must be unsigned integer", exc_info=True)
@@ -478,9 +483,6 @@ class CvnnModel:
         if not (isinstance(batch_size, int) and batch_size > 0):
             logger.error("Batch size must be unsigned integer", exc_info=True)
             sys.exit(-1)
-        if not learning_rate > 0:
-            logger.error("Learning rate must be positive", exc_info=True)
-            sys.exit(-1)
         if isinstance(display_freq, int):
             assert display_freq > 0, "display_freq must be positive"
         else:
@@ -525,7 +527,7 @@ class CvnnModel:
             logger.error("verbose datatype (" + str(type(verbose)) + ") not supported")
 
     @staticmethod
-    def _process_dataset(x, y, validation_split=0.0, validation_data=None):
+    def _process_dataset(x, y, validation_split=0.0, validation_data=None, batch_size: int = 32):
         test_dataset = None
         if isinstance(x, (list, tuple, np.ndarray)):
             if validation_data is None:
@@ -537,17 +539,17 @@ class CvnnModel:
                 y_test = y[:int(dataset_length * validation_split)]
                 if len(x_test) != 0:
                     validation_data = (np.array(x_test), np.array(y_test))
-                train_dataset = tf.data.Dataset.from_tensor_slices((x_train, y_train))
+                train_dataset = tf.data.Dataset.from_tensor_slices((x_train, y_train)).batch(batch_size=batch_size)
             else:
                 if validation_split != 0:
                     logger.warning("validation_split was given but will be ignored because "
                                    "validation_data was not None")
-                train_dataset = tf.data.Dataset.from_tensor_slices((x, y))
+                train_dataset = tf.data.Dataset.from_tensor_slices((x, y)).batch(batch_size=batch_size)
         elif isinstance(x, tf.data.Dataset):
             if y is not None:
                 logger.warning("y is ignored because x was a Dataset (and should contain the labels), "
                                "however, y was not None")
-            train_dataset = x.unbatch()
+            train_dataset = x
         else:
             logger.error("dataset type ({}) not supported".format(type(x)))
             sys.exit(-1)
@@ -571,6 +573,21 @@ class CvnnModel:
             test_dataset = (None, None)
         return train_dataset, test_dataset
 
+    @staticmethod
+    def _dataset_to_array(x, y=None):
+        if isinstance(y, (list, tuple, np.ndarray, tf.Tensor)):
+            return x, y
+        elif isinstance(x, tf.data.Dataset):
+            x_test = []
+            y_test = []
+            for element in x.unbatch():
+                x_test.append(element[0].numpy())
+                y_test.append(element[1].numpy())
+            return np.array(x_test), np.array(y_test)
+        else:
+            logger.error("dataset type ({}) not supported ".format(type(x)))
+            sys.exit(-1)
+
     @classmethod  # https://stackoverflow.com/a/2709848/5931672
     def loader(cls, f):
         return pickle.load(f)  # TODO: not yet tested
@@ -589,22 +606,24 @@ class CvnnModel:
         y_out = self.call(x)
         return tf.math.argmax(y_out, 1)
 
-    def evaluate_loss(self, x, y):
+    def evaluate_loss(self, x, y=None):
         """
         Computes the output of x and computes the loss using y
         :param x: Input of the netwotk
         :param y: Labels
         :return: loss value
         """
+        x, y = self._dataset_to_array(x, y)
         return self._apply_loss(y, self.call(x)).numpy()
 
-    def evaluate_accuracy(self, x, y):
+    def evaluate_accuracy(self, x, y=None):
         """
         Computes the output of x and returns the accuracy using y as labels
         :param x: Input of the netwotk
         :param y: Labels
         :return: accuracy
         """
+        x, y = self._dataset_to_array(x, y)
         y_pred = self.predict(x)
         if y_pred.shape == y.shape:
             y_labels = y
@@ -612,13 +631,14 @@ class CvnnModel:
             y_labels = tf.math.argmax(y, 1)
         return tf.math.reduce_mean(tf.dtypes.cast(tf.math.equal(y_pred, y_labels), tf.float64)).numpy()
 
-    def evaluate(self, x, y):
+    def evaluate(self, x, y=None):
         """
-        Compues both the loss and accuracy using "evaluate_loss" and "evaluate_accuracy"
-        :param x: Input of the netwotk
+        Computes both the loss and accuracy using "evaluate_loss" and "evaluate_accuracy"
+        :param x: Input of the network
         :param y: Labels
         :return: tuple (loss, accuracy)
         """
+        x, y = self._dataset_to_array(x, y)
         return self.evaluate_loss(x, y), self.evaluate_accuracy(x, y)
 
     def evaluate_train_and_test(self, train_x, train_y, test_x=None, test_y=None):
@@ -914,7 +934,7 @@ __author__ = 'J. Agustin BARRACHINA'
 __copyright__ = 'Copyright 2020, {project_name}'
 __credits__ = ['{credit_list}']
 __license__ = '{license}'
-__version__ = '0.2.40'
+__version__ = '0.2.49'
 __maintainer__ = 'J. Agustin BARRACHINA'
 __email__ = 'joseagustin.barra@gmail.com; jose-agustin.barrachina@centralesupelec.fr'
 __status__ = '{dev_status}'
